Package: spreadr
Type: Package
Title: Simulating Spreading Activation in a Network
Version: 0.3.0
Authors@R: c(
    person("Cynthia","Siew", role = c("aut", "cre"),
      email = "cynsiewsq@gmail.com", ),
    person(c("Dirk","U."), "Wulff", role = "ctb",
      email = "dirk.wulff@gmail.com"),
    person(c("Ning","Yuan"), "Lee", role = "ctb",
      email = "ningyuan.sg@gmail.com"))
Description: The notion of spreading activation is a prevalent metaphor in the cognitive sciences. This package provides the tools for cognitive scientists and psychologists to conduct computer simulations that implement spreading activation in a network representation. The algorithmic method implemented in 'spreadr' subroutines follows the approach described in Vitevitch, Ercal, and Adagarla (2011, Frontiers), who viewed activation as a fixed cognitive resource that could spread among nodes that were connected to each other via edges or connections (i.e., a network). See Vitevitch, M. S., Ercal, G., & Adagarla, B. (2011). Simulating retrieval from a highly clustered network: Implications for spoken word recognition. Frontiers in Psychology, 2, 369. <doi:10.3389/fpsyg.2011.00369> and Siew, C. S. Q. (2019). spreadr: A R package to simulate spreading activation in a network. Behavior Research Methods, 51, 910-929. <doi: 10.3758/s13428-018-1186-5>.
License: GPL-3
Encoding: UTF-8
Depends: Rcpp (>= 0.12.5), R (>= 2.10)
Imports: Matrix, assertthat, igraph
Suggests: dplyr, fs, ggplot2, gganimate, ggraph, gifski, knitr,
        rmarkdown, testthat (>= 3.0.0)
LazyData: true
VignetteBuilder: knitr
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-05-27 07:01:32 UTC; cynthia
Author: Cynthia Siew [aut, cre],
  Dirk U. Wulff [ctb],
  Ning Yuan Lee [ctb]
Maintainer: Cynthia Siew <cynsiewsq@gmail.com>
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
URL: https://csqsiew.github.io/spreadr/
Repository: CRAN
Date/Publication: 2026-05-30 08:30:08 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-30 11:26:23 UTC; unix
Archs: spreadr.so.dSYM
