# Generated by roxygen2: do not edit by hand

export(spreadr)
importFrom(Rcpp,sourceCpp)
importFrom(assertthat,assert_that)
importFrom(assertthat,has_name)
importFrom(assertthat,is.count)
importFrom(assertthat,is.flag)
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,is_igraph)
useDynLib(spreadr)
