# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

create_mat_t <- function(mat, a_tm1, d, retention) {
    .Call('_spreadr_create_mat_t', PACKAGE = 'spreadr', mat, a_tm1, d, retention)
}

